from ctypes import *
import numpy as np
from .corelib import _core

# Define the call signatures
_compute_mse = _core.ComputeMSE
_compute_mse.argtypes = [POINTER(c_float), POINTER(c_float), c_int, c_int ]
_compute_mse.restype = c_float

_compute_rel_mse = _core.ComputeRelMSE
_compute_rel_mse.argtypes = [POINTER(c_float), POINTER(c_float), c_int, c_int, c_float ]
_compute_rel_mse.restype = c_float

_compute_rel_mse_outlier_reject = _core.ComputeRelMSEOutlierReject
_compute_rel_mse_outlier_reject.argtypes = [POINTER(c_float), POINTER(c_float), c_int, c_int, c_float, c_float ]
_compute_rel_mse_outlier_reject.restype = c_float

def mse(img, ref):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]
    return _compute_mse(img.ctypes.data_as(POINTER(c_float)),
        ref.ctypes.data_as(POINTER(c_float)), w, h)

def relative_mse(img, ref, epsilon=0.0001):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]
    return _compute_rel_mse(img.ctypes.data_as(POINTER(c_float)),
        ref.ctypes.data_as(POINTER(c_float)), w, h, epsilon)

def relative_mse_outlier_rejection(img, ref, epsilon=0.0001, percentage=0.1):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]
    return _compute_rel_mse_outlier_reject(img.ctypes.data_as(POINTER(c_float)),
        ref.ctypes.data_as(POINTER(c_float)), w, h, epsilon, percentage)


from .corelib import _core
from ctypes import *
import numpy as np

_exposure = _core.AdjustExposure
_exposure.argtypes = (POINTER(c_float), POINTER(c_float), c_int, c_int, c_float)
_exposure.restype = None

_lin_to_srgb = _core.LinearToSrgb
_lin_to_srgb.argtypes = (POINTER(c_float), POINTER(c_float), c_int, c_int, c_int)
_lin_to_srgb.restype = None

_to_byte_img = _core.ToByteImage
_to_byte_img.argtypes = (POINTER(c_float), POINTER(c_uint8), c_int, c_int, c_int)
_to_byte_img.restype = None

_zoom = _core.ZoomWithNearestInterp
_zoom.argtypes = (POINTER(c_float), POINTER(c_float), c_int, c_int, c_int)
_zoom.restype = None

_average_to_mono = _core.RgbToMonoAverage
_average_to_mono.argtypes = (POINTER(c_float), POINTER(c_float), c_int, c_int)
_average_to_mono.restype = None

_lum_to_mono = _core.RgbToMonoLuminance
_lum_to_mono.argtypes = (POINTER(c_float), POINTER(c_float), c_int, c_int)
_lum_to_mono.restype = None

def exposure_inplace(img, exposure=0):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]
    _exposure(img.ctypes.data_as(POINTER(c_float)), img.ctypes.data_as(POINTER(c_float)), w, h, exposure)
    return img

def exposure(img, exposure=0):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]
    buf = np.zeros((h, w, 3), dtype=np.float32)
    _exposure(img.ctypes.data_as(POINTER(c_float)), buf.ctypes.data_as(POINTER(c_float)), w, h, exposure)
    return buf

def lin_to_srgb(img):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]

    if len(img.shape) == 2:
        chans = 1
    else:
        chans = img.shape[2]

    buf = np.zeros((h, w, chans), dtype=np.float32)
    _lin_to_srgb(img.ctypes.data_as(POINTER(c_float)), buf.ctypes.data_as(POINTER(c_float)), w, h, chans)
    return buf

def to_byte_image(img):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]

    if len(img.shape) == 2:
        chans = 1
    else:
        chans = img.shape[2]

    buf = np.zeros((h, w, chans), dtype=np.uint8)
    _to_byte_img(img.ctypes.data_as(POINTER(c_float)), buf.ctypes.data_as(POINTER(c_uint8)), w, h, chans)
    return buf

def zoom(img, scale: int):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]
    buf = np.zeros((h * scale, w * scale, 3), dtype=np.float32)
    _zoom(img.ctypes.data_as(POINTER(c_float)), buf.ctypes.data_as(POINTER(c_float)), w, h, scale)
    return buf

def average_color_channels(img):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]
    buf = np.zeros((h, w), dtype=np.float32)
    _average_to_mono(img.ctypes.data_as(POINTER(c_float)), buf.ctypes.data_as(POINTER(c_float)), w, h)
    return buf

def luminance(img):
    img = np.array(img, dtype=np.float32, copy=False)
    h = img.shape[0]
    w = img.shape[1]
    buf = np.zeros((h, w), dtype=np.float32)
    _lum_to_mono(img.ctypes.data_as(POINTER(c_float)), buf.ctypes.data_as(POINTER(c_float)), w, h)
    return buf


import pathlib
import platform
from ctypes import cdll

if platform.system() == "Windows":
    _dll_name = "SimpleImageIOCore.dll"
elif platform.system() == "Darwin":
    _dll_name = "libSimpleImageIOCore.dylib"
else: # Use Linux-style and hope for the best :)
    _dll_name = "libSimpleImageIOCore.so"

_libpath = str(pathlib.Path(__file__).with_name(_dll_name))
_core = cdll.LoadLibrary(_libpath)


from ctypes import *
import numpy as np
from .corelib import _core
import base64

# Define the call signatures
_write_image = _core.WriteImage
_write_image.argtypes = [POINTER(c_float), c_int, c_int, c_int, c_char_p]
_write_image.restype = None

_cache_image = _core.CacheImage
_cache_image.argtypes = [POINTER(c_int), POINTER(c_int), c_char_p]
_cache_image.restype = c_int

_copy_cached_img = _core.CopyCachedImage
_copy_cached_img.argtypes = [c_int, POINTER(c_float)]
_copy_cached_img.restype = None

_write_png_to_mem = _core.WritePngToMemory
_write_png_to_mem.argtypes = [POINTER(c_float), c_int, c_int, c_int, POINTER(c_int)]
_write_png_to_mem.restype = POINTER(c_ubyte)

_free_mem = _core.FreeMemory
_free_mem.argtypes = [POINTER(c_ubyte)]
_free_mem.restype = None

def read(filename: str):
    w = c_int()
    h = c_int()
    idx = _cache_image(byref(w), byref(h), filename.encode('utf-8'))
    buffer = np.zeros((h.value,w.value,3), dtype=np.float32)
    _copy_cached_img(idx, buffer.ctypes.data_as(POINTER(c_float)))
    return buffer

def write(filename: str, data):
    data = np.array(data, copy=False, dtype=np.float32)
    assert len(data.shape) == 3, "Only 3D arrays of shape [row, col, channel] are supported."
    h = data.shape[0]
    w = data.shape[1]
    n = data.shape[2]
    _write_image(data.ctypes.data_as(POINTER(c_float)), w, h, n, filename.encode('utf-8'))

def base64_png(data):
    data = np.array(data, copy=False, dtype=np.float32)
    assert len(data.shape) == 3, "Only 3D arrays of shape [row, col, channel] are supported."
    h = data.shape[0]
    w = data.shape[1]
    n = data.shape[2]
    numbytes = c_int()
    mem = _write_png_to_mem(data.ctypes.data_as(POINTER(c_float)), w, h, n, byref(numbytes))
    b64 = base64.b64encode(bytearray(mem[:numbytes.value]))
    _free_mem(mem)
    return b64